import hashlib
import json
import os

import threading
import time
from xml.etree import ElementTree

import openai
import requests
from flask import Flask, request, make_response

app = Flask(__name__)

appid = os.environ.get("appid")
app_sck = os.environ.get("app_sck")
wx_token = os.environ.get("wx_token")
api_key = os.environ.get("api_key")

user_msg = {}


@app.route('/pre-freeze')
def do_something():  # 执行你想要执行的函数
    while threading.active_count() > 0:
        time.sleep(1)

    return ""


@app.route('/', methods=['GET', 'POST'])
def wechat():
    if request.method == 'GET':
        # 验证服务器地址的有效性
        echostr = request.args.get('echostr')
        if verify_server_url(request):
            return make_response(echostr)
        else:
            return "Invalid Request"
    elif request.method == 'POST':
        # 处理POST请求
        xml_data = request.data
        msg = parse_message(xml_data)
        # 用户openid
        if msg["MsgType"] == "text":
            user = msg["FromUserName"]
            print(user)

            if user not in user_msg.keys():
                user_msg[user] = [0, {"role": "system", "content": "你叫oneperfect，尽量简洁的回答问题。"}]
                # 获取用户输入的信息
            user_msg[user].append({"role": "user", "content": msg["Content"]})
            # 使用多线程，防止消息堵塞
            thread = threading.Thread(target=new_send, args=(user, user_msg[user]))
            thread.start()

            return make_response("")
        return make_response(build_text_response(msg, "请先关注公众号oneperfect"))


def verify_server_url(request):
    signature = request.args.get('signature')
    timestamp = request.args.get('timestamp')
    nonce = request.args.get('nonce')
    token = wx_token  # 替换成自己的Token
    tmp_arr = [token, timestamp, nonce]
    tmp_arr.sort()
    tmp_str = ''.join(tmp_arr)
    tmp_str = hashlib.sha1(tmp_str.encode('utf-8')).hexdigest()
    if tmp_str == signature:
        return True
    else:
        return False


def parse_message(xml_data):
    root = ElementTree.fromstring(xml_data)
    msg = {}
    for child in root:
        msg[child.tag] = child.text
    return msg


def build_text_response(msg, content):
    resp_msg = f"""
        <xml>
        <ToUserName><![CDATA[{msg['FromUserName']}]]></ToUserName>
        <FromUserName><![CDATA[{msg['ToUserName']}]]></FromUserName>
        <CreateTime>{str(int(time.time()))}</CreateTime>
        <MsgType><![CDATA[text]]></MsgType>
        <Content><![CDATA[{content}]]></Content>
        </xml>
    """
    return resp_msg


def get_token(a, b):
    url = "https://api.weixin.qq.com/cgi-bin/stable_token"
    data = {
        "grant_type": "client_credential",
        "appid": a,
        "secret": b,
        "force_refresh": False
    }
    res = requests.post(url=url, json=data).json()
    return res['access_token']


def send_kf(a, b, c):
    url = "https://api.weixin.qq.com/cgi-bin/message/custom/send?access_token={}".format(a)
    headers = {"Content-Type": "application/json; charset=utf-8", "Connection": "keep-alive"}
    data = {
        "touser": b,
        "msgtype": "text",
        "text":
            {
                "content": c
            }
    }
    data = bytes(json.dumps(data, ensure_ascii=False).encode('utf-8'))
    res = requests.post(url=url, data=data, headers=headers).json()
    return res['errcode']


def get_cont(message):
    openai.api_key = api_key
    if message[0] == 0:
        message[0] = 1
        try:
            response = openai.ChatCompletion.create(
                model="gpt-3.5-turbo-0301",  # gpt-3.5-turbo-0301
                messages=message[1:],
                temperature=0.9,
                max_tokens=700,
                top_p=1,
                frequency_penalty=0,
                presence_penalty=0,
            )
            rep_cont = response.choices[0].message.content.strip()
            # 把gpt的回复添加到会话中，实现多轮对话
            message.append(response.choices[0].message.to_dict())
            # 顺便计算一下gpt-3.5的总token数
            tok = response.usage.total_tokens
            message[0] = 0
            # 如果token数大于2000，就重置会话
            if tok > 2000:

                # message.clear()
                # message.append({"role": "system", "content": "你叫oneperfect，尽量简洁的回答问题。"})
                message[0] = 3
        except:

            rep_cont = "网络连接错误"
            message[0] = 0

        return rep_cont.strip()
    elif message[0] == 3:
        return "休息一会儿再问吧"
    else:
        return "别着急，oneperfect正在思考答案呢"


def new_send(user, message):
    rep = get_cont(message)
    token = get_token(appid, app_sck)
    for i in range(len(rep) // 550 + 1):
        try:
            res = send_kf(token, user, rep[i * 550:(i + 1) * 550])
        except:
            print("发送失败")
    pass



if __name__ == '__main__':
    app.run(host="0.0.0.0", port=9000, threaded=True)


